// ======================================================================
//
// MenuButton.h
// jeff grills
//
// copyright 1997 origin systems inc
//
// ======================================================================

#include "First.h"
#include "MenuButton.h"

#include "Iff.h"

// ======================================================================

const Tag TAG_BTEV = TAG(B,T,E,V);
const Tag TAG_INT  = TAG3(I,N,T);

// ======================================================================

Timer       MenuButton::clickTimer(CONST_REAL(0.5), Timer::Seconds);
int         MenuButton::clickTimerButton;
MenuButton *MenuButton::clickTimerOwner;

// ======================================================================

MenuButton::MenuButton(MenuElement *parent, int maxWidth, int maxHeight, Iff *iff)
: MenuElement(parent, maxWidth, maxHeight, NULL)
{
	init();
	load(iff);
}

// ----------------------------------------------------------------------

MenuButton::~MenuButton(void)
{
	delete []events;
	events = NULL;
	init();
}

// ----------------------------------------------------------------------

void MenuButton::init(void)
{
	events    = NULL;
	numEvents = 0;
}

// ----------------------------------------------------------------------

void MenuButton::load(Iff *iff)
{
	int i;

	if (iff)
	{
		MenuElement::load(iff);

		// by default, MenuTextEdit and MenuButtons are tab-stops.  Unless this is forced to be OFF
		if (tabStop != TabForceOn && tabStop != TabForceOff)
			tabStop = TabOn;

		// extract ALL events
		numEvents = iff->blockCount(Iff::FlagChunk, TAG_BTEV);
		if (numEvents > 0)
		{
			events = NEW Event[numEvents];
			memset(events, '\0', sizeof(Event) * numEvents);
			for (i = 0; iff->seekChunk(TAG_BTEV, Iff::FlagOptional); i++)
			{
				events[i].button = iff->read_int32();
				events[i].type   = iff->read_int32();
				events[i].event  = iff->read_int32();
				iff->exitChunk();
			}
		}
	}
}

// ----------------------------------------------------------------------

IoResult MenuButton::event(IoEventStruct *event)
{
	const Event *e;
	int         i;

	// update the global click timer
	if (clickTimerOwner == this && event->type == IoEventMouseSetCursor && clickTimer.updateZero())
		clickTimerOwner = NULL;

	if (menuRect.isInInclusive(event->mouseX, event->mouseY))
	{
		if (event->type == IoEventMousePress && isValidButton(event->button))
		{
			clickTimerOwner = this;
			clickTimerButton = event->button;
			clickTimer.reset();

			for (e = events, i = 0; i < numEvents; e++, i++)
				if (e->button == event->button && e->type == press)
					menuEvent(e->event, this);

			return IoResultBlock;
		}

		if (event->type == IoEventMouseRelease && isValidButton(event->button))
		{
			// generate the proper release event
			for (e = events, i = 0; i < numEvents; e++, i++)
				if (e->button == event->button && e->type == release)
					menuEvent(e->event, this);

			// generate the proper click event
			if (clickTimerOwner == this)
			{
				// only generate a click if the proper button is being released
				if (clickTimerButton == event->button)
				{
					for (e = events, i = 0; i < numEvents; e++, i++)
						if (e->button == event->button && e->type == click)
							menuEvent(e->event, this);
				}

				// reset the click stuff even if it was the wrong butotn
				clickTimerOwner = NULL;
				clickTimerButton = event->button;
			}

			return IoResultBlock;
		}
	}
	else
	{
		// if any event is outside the rect, we can't be clicking
		if (clickTimerOwner == this)
			clickTimerOwner = NULL;
	}

	return MenuElement::event(event);
}

// ======================================================================
	
MenuToggle::MenuToggle(MenuElement *parent, int maxWidth, int maxHeight, Iff *iff)
: MenuButton(parent, maxWidth, maxHeight, NULL)
{
	menuResourceInteger.setOwner(this);
	load(iff);
}

// ----------------------------------------------------------------------

MenuToggle::~MenuToggle(void)
{
}

// ----------------------------------------------------------------------

void MenuToggle::load(Iff *iff)
{
	if (iff)
	{
		MenuButton::load(iff);

		if (iff->seekForm(TAG_INT, Iff::FlagOptional))
		{
			menuResourceInteger.load(iff);
			iff->exitForm();
		}
	}
}	

// ----------------------------------------------------------------------

MenuResourceInteger *MenuToggle::getMenuResourceInteger(void)
{
	return &menuResourceInteger;
}

// ----------------------------------------------------------------------

void MenuToggle::menuEvent(int id, MenuElement *whom)
{
	MenuResourceInteger *i;

	MenuButton::menuEvent(id, whom);

	if ((i = getMenuResourceInteger()) != NULL)
		i->menuEvent(id, whom);
}

// ----------------------------------------------------------------------

void MenuToggle::menuDraw(int x, int y) const
{
	MenuButton::menuDraw(x, y);
}

// ======================================================================
